/******************************************************************************\
* Copyright (C) 2005 by Relcom Kft.
* All rights reserved.
*------------------------------------------------------------------------------
* PROJECT.......... Board Support Library for pcDSP6
* VERSION.......... 1.0
* COMPILER......... Code Composer Studio 3.0
*------------------------------------------------------------------------------
* CONTENT.......... Main header file of Board Support Library for pcDSP6
* FILENAME......... bsl_bsl.h
* LAST MODIFIED AT. 11/04/2005
*------------------------------------------------------------------------------
*
* This header file should be included by the main header file of all the
* projects that applies the BSL.
*
\******************************************************************************/
#ifndef _BSL_BSL_H_
#define _BSL_BSL_H_


#ifndef _INLINE
	#define _INLINE
#endif

// Define the type of the DSP chip fot the CSL (TI's Chip Support Library)
#define CHIP_6416	1

#include <csl_stdinc.h>
#include <csl_timer.h>
#include <bsl_stdinchal.h>


/******************************************************************************\
* scope and inline control macros
\******************************************************************************/
#ifdef __cplusplus
	#define BSLAPI extern "C" far
#else
	#define BSLAPI extern far
#endif

#undef	USEDEFS
#undef	IDECL
#undef	IDEF

#ifdef	_BSL_BSL_MOD_
	#define IDECL BSLAPI
	#define USEDEFS
	#define IDEF
#else
	#ifdef	_INLINE
		#define	IDECL static inline
		#define	USEDEFS
		#define	IDEF  static inline
	#else
		#define	IDECL BSLAPI
	#endif
#endif


/******************************************************************************\
* global macro declarations
\******************************************************************************/

// Enable CPLD- and FPGA-realized periperals
#define BSL_AOUT_SUPPORT			1
#define BSL_BBAIO_SUPPORT			1
#define BSL_BDAC_SUPPORT			1
#define BSL_FPGA_SUPPORT			1
#define BSL_HPCI_SUPPORT			1
#define BSL_MISC_SUPPORT			1
#define BSL_PCI_SUPPORT				1
#define BSL_MISC_SUPPORT			1
#define BSL_UBADC_SUPPORT			1
#define BSL_UBDAC_SUPPORT			1
#define BSL_DDSREG_SUPPORT			1
#define BSL_UDDS_SUPPORT			1
#define BSL_UFRMR_SUPPORT			1
#define BSL_UHADC_SUPPORT			1
#define BSL_UINT_SUPPORT			1
#define BSL_UMISC_SUPPORT			1
#define BSL_UMLVDSB_SUPPORT			1
#define BSL_USBUS_SUPPORT			1
// Enable other software peripherals
#define BSL_PCIBUS_SUPPORT			1

// invalid function
#define INVALID_VFXN				((VFxn)(0xFFFFFFFF))

// DSP processor's nominal speed in MHz
#define DSP_NOMINAL_SPEED_IN_MHZ	1000

//
// System-wide free-running 8 nanosec time-base counter, realized by Timer 2.
//

// Defines a system-wide free-running 8 nanosec counter. It is ticked by the
// Timer 2 peripheral. Interrupt is not enabled for the timer. Only the counter
// register of the timer is used as a free-running counter.
// Features:
// - 32-bit counter
// - time-base is 8 nanosec
// - overround-time is 34.35 sec

// Time-base in nanosec
#define BSL_FRC_TIME_BASE_IN_NSEC	8

// Provides the current value of the free-running counter
#define BSL_FRC_GET_COUNT			TIMER_RGET(CNT2)

// Converts the given microsec to counter ticks
#define BSL_FRC_CONV_NSEC_TO_TICKS(nanosec) 			\
	(Uint32)((((Uint32)(nanosec)) +						\
		(((Uint32)BSL_FRC_TIME_BASE_IN_NSEC) - 1)) /	\
		((Uint32)BSL_FRC_TIME_BASE_IN_NSEC))

// Sets the given 32-bit variable to the current value of the counter
#define BSL_FRC_START(start_var32)	((start_var32) = BSL_FRC_GET_COUNT)

// Returns TRUE, if the counter has expired. (Period must be set in ticks.)
#define BSL_FRC_IS_EXPIRED_TICK(start_var32, period_in_ticks) 	\
	((BSL_FRC_GET_COUNT - ((Uint32)(start_var32))) >= ((Uint32)(period_in_ticks)))

// Returns TRUE, if the counter has expired. (Period must be set in nanosec.)
#define BSL_FRC_IS_EXPIRED_NSEC(start_var32, period_in_nsec) 	\
	BSL_FRC_IS_EXPIRED_TICK(start_var32,BSL_FRC_CONV_NSEC_TO_TICKS(period_in_nsec))

// Delays the given nanosec
#define BSL_FRC_DELAY_NANOSEC(nanosec)									\
	do {																\
		Uint32	TimerForDelay;											\
		Uint32	PerionInTicks;											\
		BSL_FRC_START( TimerForDelay );									\
		PerionInTicks = BSL_FRC_CONV_NSEC_TO_TICKS( nanosec );			\
		while(!BSL_FRC_IS_EXPIRED_TICK(TimerForDelay,PerionInTicks));	\
	} while(0)


/******************************************************************************\
* global typedef declarations
\******************************************************************************/

typedef double Double;

typedef void (*VFxn)();

typedef Uint32 BOOL;


/******************************************************************************\
* global variable declarations
\******************************************************************************/


/******************************************************************************\
* global function declarations
\******************************************************************************/

BSLAPI void BSL_init();


/******************************************************************************\
* inline function declarations
\******************************************************************************/


/******************************************************************************\
* inline function definitions
\******************************************************************************/
#ifdef USEDEFS


#endif /* USEDEFS */

#endif /* _BSL_BSL_H_ */
/******************************************************************************\
* End of bsl_bsl.h
\******************************************************************************/

