/******************************************************************************\
* Copyright (C) 2005 by Relcom Kft.
* All rights reserved.
*------------------------------------------------------------------------------
* PROJECT.......... Board Support Library for pcDSP6
* VERSION.......... 1.0
* COMPILER......... Code Composer Studio 3.0
*------------------------------------------------------------------------------
* CONTENT.......... Interface for DDS chip of pcDSP6
* FILENAME......... bsl_dds.h
* LAST MODIFIED AT. 01/22/2006
\******************************************************************************/

#ifndef _BSL_DDS_H_
#define _BSL_DDS_H_

#include <bsl.h>
#include <bsl_ddsio.h>
#include <bsl_ddsreghal.h>

/******************************************************************************\
* scope and inline control macros
\******************************************************************************/
#ifdef	__cplusplus
	#define	BSLAPI extern "C" far
#else
	#define	BSLAPI extern far
#endif

#undef	USEDEFS
#undef	IDECL
#undef	IDEF

#ifdef	_BSL_DDS_MOD_
	#define IDECL BSLAPI
	#define USEDEFS
	#define IDEF
#else
	#ifdef	_INLINE
		#define	IDECL static inline
		#define	USEDEFS
		#define	IDEF  static inline
	#else
		#define	IDECL BSLAPI
	#endif
#endif


/******************************************************************************\
* global macro declarations
\******************************************************************************/

//! Writes all bytes of the specified register.
#define DDS_WRITE_REG(regName,dataWord)	\
	DDSIO_writeRegister(				\
		_DDSREG_##regName##_ADDR,		\
		_DDSREG_##regName##_BYTES,		\
		dataWord)

//! Reads all bytes of the specified register.
#define DDS_READ_REG(regName)			\
	DDSIO_readRegister(					\
		_DDSREG_##regName##_ADDR,		\
		_DDSREG_##regName##_BYTES)		\

//! Writes the CFR1 register.
#define DDS_WRITE_CFR1(dataWord)		DDS_WRITE_REG(CFR1,dataWord)

//! Reads the CFR1 register.
#define DDS_READ_CFR1()					DDS_READ_REG(CFR1)

//! Writes the CFR2 register.
#define DDS_WRITE_CFR2(dataWord)		DDS_WRITE_REG(CFR2,dataWord)

//! Reads the CFR2 register.
#define DDS_READ_CFR2()					DDS_READ_REG(CFR2)

//! Writes the ASF register.
#define DDS_WRITE_ASF(dataWord)			DDS_WRITE_REG(ASF,dataWord)

//! Reads the ASF register.
#define DDS_READ_ASF()					DDS_READ_REG(ASF)

//! Writes the ARR register.
#define DDS_WRITE_ARR(dataWord)			DDS_WRITE_REG(ARR,dataWord)

//! Reads the ARR register.
#define DDS_READ_ARR()					DDS_READ_REG(ARR)

//! Writes the FTW0 register.
#define DDS_WRITE_FTW0(dataWord)		DDS_WRITE_REG(FTW0,dataWord)

//! Reads the FTW0 register.
#define DDS_READ_FTW0()					DDS_READ_REG(FTW0)

//! Writes the POW0 register.
#define DDS_WRITE_POW0(dataWord)		DDS_WRITE_REG(POW0,dataWord)

//! Reads the POW0 register.
#define DDS_READ_POW0()					DDS_READ_REG(POW0)

//! 
#define DDS_UPDATE()

/******************************************************************************\
* global typedef declarations
\******************************************************************************/


/******************************************************************************\
* global variable declarations
\******************************************************************************/


/******************************************************************************\
* global function declarations
\******************************************************************************/

//- ----------------------------------------------------------------------------
//! \brief	Resets the DDS chip's settings.
//!
//!	Resets the DDS chip's settings. Clears all the registers to zeros.
//!
//!	\return	Void.
//- ----------------------------------------------------------------------------
BSLAPI void DDS_reset(void);

//- ----------------------------------------------------------------------------
//! \brief	Updates the DDS chip's registers.
//!
//!	Updates the DDS chip's registers.
//!
//!	\return	Void.
//- ----------------------------------------------------------------------------
BSLAPI void DDS_update(void);

//- ----------------------------------------------------------------------------
//! \brief	Set the closest output frequency to the desired one.
//!
//!	Set the closest output frequency to the desired one. Writes the FTW0 reg.,
//!	but the DDS chip is not updated.
//!
//!	\return	The set output frequency in fact in Hz.
//- ----------------------------------------------------------------------------
BSLAPI double DDS_setOutputFreq(
	double desiredOutputFreqHz);	//!< desired output frequency in Hz

/******************************************************************************\
* inline function declarations
\******************************************************************************/


/******************************************************************************\
* inline function definitions
\******************************************************************************/
#ifdef USEDEFS

/*----------------------------------------------------------------------------*/
#endif /* USEDEFS */


#endif /* _BSL_DDS_H_ */
/******************************************************************************\
* End of bsl_dds.h
\******************************************************************************/

