/******************************************************************************\
* Copyright (C) 2005 by Relcom Kft.
* All rights reserved.
*------------------------------------------------------------------------------
* PROJECT.......... Board Support Library for pcDSP6
* VERSION.......... 1.0
* COMPILER......... Code Composer Studio 3.0
*------------------------------------------------------------------------------
* CONTENT.......... Serial Input/Output Interface for DDS chip of pcDSP6
* FILENAME......... bsl_ddsio.h
* LAST MODIFIED AT. 01/22/2006
\******************************************************************************/

#ifndef _BSL_DDSIO_H_
#define _BSL_DDSIO_H_

#include <bsl.h>
#include <bsl_uddshal.h>

/******************************************************************************\
* scope and inline control macros
\******************************************************************************/
#ifdef	__cplusplus
	#define	BSLAPI extern "C" far
#else
	#define	BSLAPI extern far
#endif

#undef	USEDEFS
#undef	IDECL
#undef	IDEF

#ifdef	_BSL_DDSIO_MOD_
	#define IDECL BSLAPI
	#define USEDEFS
	#define IDEF
#else
	#ifdef	_INLINE
		#define	IDECL static inline
		#define	USEDEFS
		#define	IDEF  static inline
	#else
		#define	IDECL BSLAPI
	#endif
#endif


/******************************************************************************\
* global macro declarations
\******************************************************************************/


/******************************************************************************\
* global typedef declarations
\******************************************************************************/


/******************************************************************************\
* global variable declarations
\******************************************************************************/


/******************************************************************************\
* global function declarations
\******************************************************************************/

//- ----------------------------------------------------------------------------
//! \brief	Sets the DDSIO bus wires in default state.
//!
//!	Sets the DDSIO bus wires in default state:
//!		1. Chip Select is high.
//!		2. SDIO is driven by the FPGA.
//!		3. Clock is low.
//!		4. Data is low.
//!
//!	\return	Void.
//- ----------------------------------------------------------------------------
BSLAPI void DDSIO_reset(void);

//- ----------------------------------------------------------------------------
//! \brief	Writes the specified register with the given number bytes of a
//!			32-bit word.
//!
//! \pre	Lastly, a RESET or STOP condition was performed.
//!
//!	Writes the specified register with the given number bytes of a 32-bit word.
//!
//!	\post	STOP condition is performed.
//!
//!	\return	Void.
//- ----------------------------------------------------------------------------
BSLAPI void DDSIO_writeRegister(
	Uint32 regAddr,		//!< address of the register
	Uint32 numOfBytes,	//!< number of bytes to be written
	Uint32 dataWord);	//!< bytes of the up to 32-bit word to be written

//- ----------------------------------------------------------------------------
//! \brief	Reads the given number bytes of the specified register.
//!
//! \pre	Lastly, a RESET or STOP condition was performed.
//!
//!	Reads the given number bytes of the specified register.
//!
//!	\post	STOP condition is performed.
//!
//!	\return	The read bytes of the specified register.
//- ----------------------------------------------------------------------------
BSLAPI Uint32 DDSIO_readRegister(
	Uint32 regAddr,		//!< address of the register
	Uint32 numOfBytes);	//!< number of bytes to be written


/******************************************************************************\
* inline function declarations
\******************************************************************************/


/******************************************************************************\
* inline function definitions
\******************************************************************************/
#ifdef USEDEFS

/*----------------------------------------------------------------------------*/
#endif /* USEDEFS */


#endif /* _BSL_DDSIO_H_ */
/******************************************************************************\
* End of bsl_ddsio.h
\******************************************************************************/

