/******************************************************************************\
* Copyright (C) 2005 by Relcom Kft.
* All rights reserved.
*------------------------------------------------------------------------------
* PROJECT.......... Board Support Library for pcDSP6
* VERSION.......... 1.0
* COMPILER......... Code Composer Studio 3.0
*------------------------------------------------------------------------------
* CONTENT.......... Header file for API of High-speed A/D
* FILENAME......... bsl_hadc.h
* LAST MODIFIED AT. 11/04/2005
\******************************************************************************/
#ifndef _BSL_HADC_H_
#define _BSL_HADC_H_

#include <bsl.h>
#include <bsl_uhadchal.h>
#include <csl_gpiohal.h>


/******************************************************************************\
* scope and inline control macros
\******************************************************************************/
#ifdef __cplusplus
	#define BSLAPI extern "C" far
#else
	#define BSLAPI extern far
#endif

#undef	USEDEFS
#undef	IDECL
#undef	IDEF

#ifdef	_BSL_HADC_MOD_
	#define IDECL BSLAPI
	#define USEDEFS
	#define IDEF
#else
	#ifdef	_INLINE
		#define	IDECL static inline
		#define	USEDEFS
		#define	IDEF  static inline
	#else
		#define	IDECL BSLAPI
	#endif
#endif


/******************************************************************************\
* global macro declarations
\******************************************************************************/

// Write enable is controlled via GPIO8
#define HADC_WRITE_ENABLE_GPIO_BIT			8

// Bitmask for GPIO8
#define HADC_WRITE_ENABLE_GPIO_BITMASK		\
	((Uint32)(((Uint32)1) << ((Uint32)(HADC_WRITE_ENABLE_GPIO_BIT))))

// Enable FIFO write
#define HADC_ENABLE_FIFO_WRITE()				\
	(GPIO_RSET( GPVAL, GPIO_RGET(GPVAL) | HADC_WRITE_ENABLE_GPIO_BITMASK))

// Disable FIFO write
#define HADC_DISABLE_FIFO_WRITE()				\
	(GPIO_RSET( GPVAL, GPIO_RGET(GPVAL) & (~HADC_WRITE_ENABLE_GPIO_BITMASK) ))

// Read HAD1's FIFO
#define _UHADC_FIFO1RD_ADDR			0x6C000000u
#define HADC_READ_FIFO1()			_PER_RGET16(_UHADC_FIFO1RD_ADDR,UHADC,FIFO1RD)

// Read HAD2's FIFO
#define _UHADC_FIFO2RD_ADDR			0x6C004000u
#define HADC_READ_FIFO2()			_PER_RGET16(_UHADC_FIFO2RD_ADDR,UHADC,FIFO2RD)

// TRUE, if FIFO1 is empty
#define HADC_IS_FIFO1_EMPTY()		UHADC_FGETSQ(FIFOST, EMPTY1, EMPTY)

// TRUE, if FIFO2 is empty
#define HADC_IS_FIFO2_EMPTY()		UHADC_FGETSQ(FIFOST, EMPTY2, EMPTY)

// TRUE, if FIFO1 is full
#define HADC_IS_FIFO1_FULL()		UHADC_FGETSQ(FIFOST, FULL1, FULL)

// TRUE, if FIFO2 is full
#define HADC_IS_FIFO2_FULL()		UHADC_FGETSQ(FIFOST, FULL2, FULL)


/******************************************************************************\
* global typedef declarations
\******************************************************************************/


/******************************************************************************\
* global variable declarations
\******************************************************************************/


/******************************************************************************\
* global function declarations
\******************************************************************************/

//- ----------------------------------------------------------------------------
//! \brief	Initializes the HADC module.
//!
//!	Initializes the HADC module.
//! Must be called from the BSL_init().
//!
//!	\return	Void.
//- ----------------------------------------------------------------------------
BSLAPI void HADC_init();


/******************************************************************************\
* inline function declarations
\******************************************************************************/


/******************************************************************************\
* inline function definitions
\******************************************************************************/
#ifdef USEDEFS


#endif /* USEDEFS */

#endif /* _BSL_HADC_H_ */
/******************************************************************************\
* End of bsl_hadc.h
\******************************************************************************/

