/******************************************************************************\
* Copyright (C) 2006 by Relcom Kft.
* All rights reserved.
*------------------------------------------------------------------------------
* PROJECT.......... Board Support Library for pcDSP6
* VERSION.......... 1.0
* COMPILER......... Code Composer Studio 3.0
*------------------------------------------------------------------------------
* CONTENT.......... Header file for API of peripheral PCI Bus
* FILENAME......... bsl_pcibus.h
* LAST MODIFIED AT. 26/02/2006
\******************************************************************************/
#ifndef _BSL_PCIBUS_H_
#define _BSL_PCIBUS_H_

#include <bsl.h>
#include <csl_pci.h>


#if (BSL_PCIBUS_SUPPORT)
/******************************************************************************\
* scope and inline control macros
\******************************************************************************/
#ifdef	__cplusplus
	#define	BSLAPI extern "C" far
#else
	#define	BSLAPI extern far
#endif

#undef	USEDEFS
#undef	IDECL
#undef	IDEF

#ifdef	_BSL_PCIBUS_MOD_
	#define IDECL BSLAPI
	#define USEDEFS
	#define IDEF
#else
	#ifdef	_INLINE
		#define	IDECL static inline
		#define	USEDEFS
		#define	IDEF  static inline
	#else
		#define	IDECL BSLAPI
	#endif
#endif


/******************************************************************************\
* global macro declarations
\******************************************************************************/

// transfer directions
// (direct 'Start' field values for PCI Master Control Register)
#define PCIBUS_TRANSFER_FROM_DSP			PCI_PCIMC_START_WRITE
#define PCIBUS_TRANSFER_TO_PCI				PCI_PCIMC_START_WRITE

#define PCIBUS_TRANSFER_TO_DSP				PCI_PCIMC_START_READPREF
#define PCIBUS_TRANSFER_FROM_PCI			PCI_PCIMC_START_READPREF

// In the PCI Master Control Register, there is a reserved bit field
// from bit 3 to bit 15. This region of bits is used to store control
// status bits for PCI Module.

#define PCIBUS_AUTOINCREMENT_ON				(0x00000000u)
#define PCIBUS_AUTOINCREMENT_OFF			(0x00000008u)
#define PCIBUS_AUTOINCREMENT_ENABLE			(0x00000000u)
#define PCIBUS_AUTOINCREMENT_DISABLE		(0x00000008u)

// Mask for "Type Of Error" values
#define PCIBUS_TOE_MASK						(0x0000FF00u)
#define PCIBUS_TOE_MASK_INV					(0xFFFF00FFu)

// "Type Of Error" values
#define PCIBUS_TOE_SUCCESSFUL				(0x00000000u)
#define PCIBUS_TOE_NO_ERROR					(0x00000000u)
#define PCIBUS_TOE_HARDWARE_BUSY			(0x00000100u)
#define PCIBUS_TOE_TRANSFER_IN_PROCESS		(0x00000200u)
#define PCIBUS_TOE_TIMEOUT					(0x00000400u)
#define PCIBUS_TOE_REC_TARGET_ABORT			(0x00002000u)
#define PCIBUS_TOE_REC_MASTER_ABORT			(0x00004000u)
#if !(C64_SUPPORT)
#define PCIBUS_TOE_DMAHALTED				(0x00008000u)
#endif

#define PCIBUS_ISR_COMPLETED				1
#define PCIBUS_ISR_FAILED					2
#define PCIBUS_ISR_BUSY						0


/******************************************************************************\
* global typedef declarations
\******************************************************************************/

// Access Request Object
// Descriptor to define a data transfer between an internal DSP memory buffer
// and the Expansion Bus of the DSP processor.
typedef struct
{
	Uint32			regDSPMA;	// DSP Master Address reg.
	Uint32			regPCIMA;	// PCI Master Address reg.
	Uint32			regPCIMC;	// PCI Master Control reg. + PCI Module Control
	volatile BOOL	completed;
} PCIBUS_AccessReqObj;


/******************************************************************************\
* global variable declarations
\******************************************************************************/


/******************************************************************************\
* global function declarations
\******************************************************************************/

void PCIBUS_init();

//==============================================================================
// Returns the status of the current PCI Bus transaction.
//
// returns:
//		PCIBUS_ISR_COMPLETED		Completed successfully.
//		PCIBUS_ISR_FAILED			Failed.
//		PCIBUS_ISR_BUSY				Is still in progress.
//==============================================================================
Uint32 PCIBUS_isISRAccessComplete();

//==============================================================================
//	Fills up an PCIBUS_AccessReqObj type access request object to define a data
//	transfer between an internal DSP memory buffer and the PCI Bus of the DSP
//	processor.
//
//	In order to request the defined access, call the PCIBUS_accessReq() function.
//
//	parameters:
//		pReqObj				pointer to a PCIBUS_AccessReqObj type access request
//							object
//							The object has to be unmodified until the request
//							has been executed.
//		pInternalBuffer		32-bit unsigned integer type pointer to the internal
//							buffer that is in the DSP's (or DSP board's) memory
//							This buffer can be either the source or the
//							destination of the data transfer according to the
//							direction of the transfer.
//		pciAddress			32-bit PCI Memory address
//							This address is the starting address of either the
//							source or the destination of the data transfer
//							according to the direction of the transfer.
//		lengthIn32bWs		16-bit unsigned integer value for number of the
//							bytes to be transferred (copied)
//		trDir_autoIncr		selects the direction of the data transfer and
//							enables/disables the address autoincrement in the
//							DSP. Use constants from PCIBUS_TRANSFER_x and from
//							PCIBUS_AUTOINCREMENT_x OR-ed.
//==============================================================================
BSLAPI void PCIBUS_makeReq_Mem( PCIBUS_AccessReqObj * pReqObj,
	Uint32 * pInternalBuffer, Uint32 pciAddress,
	Uint16 lengthIn32bWs, Uint32 trDir_autoIncr);

//==============================================================================
//	Fills up an PCIBUS_AccessReqObj type access request object to define a data
//	transfer between an internal DSP memory buffer and the PCI Bus of the DSP
//	processor.
//
//	In order to request the defined access, call the PCIBUS_accessReq() function.
//
//	parameters:
//		pReqObj				pointer to a PCIBUS_AccessReqObj type access request
//							object
//							The object has to be unmodified until the request
//							has been executed.
//		val					pointer to an Uint32-type variable that is the
//							source or the destination of the transfer according
//							to the direction of the transfer
//		pciAddress			16-bit PCI I/O address
//							This address is the starting address of either the
//							source or the destination of the data transfer
//							according to the direction of the transfer.
//		trDir				selects the direction of the data transfer
//							Use constants from PCIBUS_TRANSFER_x.
//==============================================================================
BSLAPI void PCIBUS_makeReq_IO( PCIBUS_AccessReqObj * pReqObj,
	Uint32 *val, Uint32 pciAddress, Uint32 trDir);

//==============================================================================
// Fills up a PCIBUS_AccessReqObj type access request object to define a single
// 32-bit word read/write by a PCI Configuration I/O Cycle.
//
// In order to request the defined access, call the PCIBUS_accessReq() function.
//
// parameters:
//		pReqObj				pointer to a PCIBUS_AccessReqObj type access request
//							object
//							The object has to be unmodified until the request
//							has been executed.
//		reg					ordinal number of the register to be reached
//		func				PCI function number of the PCI device to be reached
//		dev					PCI device number of the PCI device to be reached
//		val					pointer to an Uint32-type variable that is the
//							source or the destination of the transfer according
//							to the direction of the transfer
//		transferDirection	select the direction of the data transfer
//							constants from PCIBUS_TRANSFER_x
//==============================================================================
BSLAPI void PCIBUS_makeReq_Cfg( PCIBUS_AccessReqObj * pReqObj,
	Uint32 reg, Uint32 func, Uint32 dev, Uint32 *val, Uint32 trDir);

//==============================================================================
//	Places the previously filled up PCIBUS_AccessReqObj type access request
//	object at the end of the given queue of the access requests.
//
//	In order to fill up a PCIBUS_AccessReqObj type access request object, call
//	the PCIBUS_makeReq() function.
//
//	parameters:
//		pReqObj			pointer for a PCIBUS_AccessReqObj type access request
//						object that has been filled up previously
//==============================================================================
BSLAPI BOOL PCIBUS_accessReq( PCIBUS_AccessReqObj * pReqObj);

//==============================================================================
// If the queue is empty and there is not currently a PCI transaction in
// progress, starts the defined request without interrupt.
//
// In order to fill up a PCIBUS_AccessReqObj type access request object, call
// the PCIBUS_makeReq() function.
// The request object can be temporary.
//
// parameters:
//		pReqObj			pointer for a PCIBUS_AccessReqObj type access request
//						object that has been filled up previously
//
// returns:
//		TRUE			PCI access has been started.
//		FALSE			There is a PCI access request in progress.
//==============================================================================
BSLAPI BOOL PCIBUS_accessReqFromISR( PCIBUS_AccessReqObj * pReqObj);

//==============================================================================
//	Places the previously filled up PCIBUS_AccessReqObj type access request
//	objects at the end of the given queue of the access requests after each
//	other directly.
//
//	In order to fill up a PCIBUS_AccessReqObj type access request object, call
//	the PCIBUS_makeReq() function.
//
//	parameters:
//		pReqObjArray	pointer for the array of PCIBUS_AccessReqObj type access
//						request objects that have been filled up previously
//		numberOfReqs	number of elements in the array above
//==============================================================================
BSLAPI void PCIBUS_accessMoreReq(
	PCIBUS_AccessReqObj * pReqObjArray, Uint32 numberOfReqs);

//==============================================================================
//	Executes the previously filled up PCIBUS_AccessReqObj request.
//	Waits for the request to complete before returning
//
//	In order to fill up a PCIBUS_AccessReqObj type access request object, call
//	the PCIBUS_makeReq() function.
//
//	parameters:
//		pReqObj			pointer for a PCIBUS_AccessReqObj type access request
//						object that has been filled up previously
//==============================================================================
BSLAPI BOOL PCIBUS_accessReqWithWait( PCIBUS_AccessReqObj * pReqObj );

/******************************************************************************\
* inline function declarations
\******************************************************************************/


//==============================================================================
//	Defines an Expansion Bus read according to the given parameters, requests
//	the access, and then waits the end of the defined data transfer.
//
//	parameters:
//		pInternalBuffer		32-bit unsigned integer type pointer to the internal
//							buffer that is in the DSP's (or DSP board's) memory
//							This buffer is the destination of the data transfer.
//		pciAddress			32-bit Expansion Bus address
//							This address is the starting address of the source
//							of the data transfer.
//		lengthIn32bWs		16-bit unsigned integer value for number of the
//							32-bit words to be transferred (copied)
//
//	return:
//		error codes:
//		PCIBUS_TOE_NO_ERROR			= no error happened
//		PCIBUS_TOE_REC_TARGET_ABORT	= PCI Receiver Target Abort Error
//		PCIBUS_TOE_REC_MASTER_ABORT	= PCI Receiver Master Abort Error
//==============================================================================
IDECL Uint32 PCIBUS_read(Uint32 * pInternalBuffer, Uint32 pciAddress,
	Uint16 lengthIn32bWs);
IDECL Uint32 PCIBUS_readFromISR_NoWait(Uint32 * pInternalBuffer, Uint32 pciAddress,
	Uint16 lengthIn32bWs);

//==============================================================================
//	Defines an Expansion Bus write according to the given parameters, requests
//	the access, and then waits the end of the defined data transfer.
//
//	parameters:
//		pInternalBuffer		32-bit unsigned integer type pointer to the internal
//							buffer that is in the DSP's (or DSP board's) memory
//							This buffer is the source of the data transfer.
//		pciAddress		32-bit Expansion Bus address
//							This address is the starting address of the destination
//							of the data transfer.
//		lengthIn32bWs		16-bit unsigned integer value for number of the
//							32-bit words to be transferred (copied)
//
//	return:
//		error codes:
//		PCIBUS_TOE_NO_ERROR			= no error happened
//		PCIBUS_TOE_REC_TARGET_ABORT	= PCI Receiver Target Abort Error
//		PCIBUS_TOE_REC_MASTER_ABORT	= PCI Receiver Master Abort Error
//==============================================================================
IDECL Uint32 PCIBUS_write(Uint32 * pInternalBuffer, Uint32 pciAddress,
	Uint16 lengthIn32bWs);
IDECL Uint32 PCIBUS_writeFromISR_NoWait(Uint32 * pInternalBuffer, Uint32 pciAddress,
	Uint16 lengthIn32bWs);

//==============================================================================
//	Defines an Expansion Bus data transfer to read a single 32-bit word
//	according to the given parameter, requests the access, and then waits the
//	 end of the defined data transfer.
//
//	parameters:
//		pciAddress		32-bit Expansion Bus address
//							This address will be read.
//		value			pointer to the 32-bit word to be written by the read
//							value
//
//	return:
//		error codes:
//		PCIBUS_TOE_NO_ERROR			= no error happened
//		PCIBUS_TOE_REC_TARGET_ABORT	= PCI Receiver Target Abort Error
//		PCIBUS_TOE_REC_MASTER_ABORT	= PCI Receiver Master Abort Error
//==============================================================================
IDECL Uint32 PCIBUS_readOneWord(Uint32 pciAddress, Uint32 *value);

//==============================================================================
//	Defines an Expansion Bus data transfer to write a single 32-bit word
//	according to the given parameters, requests the access, and then waits the
//	end of the defined data transfer.
//
//	Callable from TSK only.
//
//	parameters:
//		pciAddress		32-bit Expansion Bus address
//							This address will be written.
//		value			new 32-bit value of the memory at the given address
//
//	return:
//		error codes:
//		PCIBUS_TOE_NO_ERROR			= no error happened
//		PCIBUS_TOE_REC_TARGET_ABORT	= PCI Receiver Target Abort Error
//		PCIBUS_TOE_REC_MASTER_ABORT	= PCI Receiver Master Abort Error
//==============================================================================
IDECL Uint32 PCIBUS_writeOneWord(Uint32 pciAddress, Uint32 value);

//==============================================================================
//	Returns the error code of an Expansion Bus access.
//
//	parameters:
//		pReqObj				pointer to the PCIBUS_AccessReqObj type access
//							request object that has already been executed
//
//	return:
//		error codes:
//		PCIBUS_TOE_NO_ERROR			= no error happened
//		PCIBUS_TOE_REC_TARGET_ABORT	= PCI Receiver Target Abort Error
//		PCIBUS_TOE_REC_MASTER_ABORT	= PCI Receiver Master Abort Error
//==============================================================================
IDECL Uint32 PCIBUS_getError(PCIBUS_AccessReqObj * pReqObj);

//==============================================================================
//	Returns whether or not the data transfer has completed
//
//	parameters:
//		pReqObj				pointer to the PCIBUS_AccessReqObj type access
//							request object that is in process
//
//	return:
//		TRUE				The data transfer has completed.
//		FALSE				The data transfer is in process.
//==============================================================================
IDECL BOOL PCIBUS_isCompleted(PCIBUS_AccessReqObj * pReqObj);


/******************************************************************************\
* inline function definitions
\******************************************************************************/
#ifdef USEDEFS

/*----------------------------------------------------------------------------*/
/* You can call this function from a TSK only.
 * Do *NOT* call this function from the main(), or SWI (PRD, CLK), or HWI.
 */
IDEF Uint32 PCIBUS_read(Uint32 * pInternalBuffer, Uint32 pciAddress,
	Uint16 lengthIn32bWs)
{
	PCIBUS_AccessReqObj	reqObj;

	PCIBUS_makeReq_Mem(&reqObj, pInternalBuffer, pciAddress, lengthIn32bWs,
		PCIBUS_TRANSFER_TO_DSP | PCIBUS_AUTOINCREMENT_ON);
	PCIBUS_accessReqWithWait(&reqObj);

	return PCIBUS_getError(&reqObj);
}

IDEF Uint32 PCIBUS_readFromISR_NoWait(Uint32 * pInternalBuffer, Uint32 pciAddress,
	Uint16 lengthIn32bWs)
{
	PCIBUS_AccessReqObj	reqObj;

	PCIBUS_makeReq_Mem(&reqObj, pInternalBuffer, pciAddress, lengthIn32bWs,
		PCIBUS_TRANSFER_TO_DSP | PCIBUS_AUTOINCREMENT_ON);
	
	// start the PCI access and get error, if there
	return PCIBUS_accessReqFromISR(&reqObj);
}

/*----------------------------------------------------------------------------*/
/* You can call this function from a TSK only.
 * Do *NOT* call this function from the main(), or SWI (PRD, CLK), or HWI.
 */
IDEF Uint32 PCIBUS_write(Uint32 * pInternalBuffer, Uint32 pciAddress,
	Uint16 lengthIn32bWs)
{
	PCIBUS_AccessReqObj	reqObj;

	PCIBUS_makeReq_Mem(&reqObj, pInternalBuffer, pciAddress, lengthIn32bWs,
		PCIBUS_TRANSFER_FROM_DSP | PCIBUS_AUTOINCREMENT_ON);
	PCIBUS_accessReqWithWait( &reqObj );

	return PCIBUS_getError(&reqObj);
}

IDEF Uint32 PCIBUS_writeFromISR_NoWait(Uint32 * pInternalBuffer, Uint32 pciAddress,
	Uint16 lengthIn32bWs)
{
	PCIBUS_AccessReqObj	reqObj;

	PCIBUS_makeReq_Mem(&reqObj, pInternalBuffer, pciAddress, lengthIn32bWs,
		PCIBUS_TRANSFER_FROM_DSP | PCIBUS_AUTOINCREMENT_ON);
	
	// start the PCI access and get error, if there
	return PCIBUS_accessReqFromISR( &reqObj );
}

/*----------------------------------------------------------------------------*/
/* You can call this function from a TSK only.
 * Do *NOT* call this function from the main(), or SWI (PRD, CLK), or HWI.
 */
IDEF Uint32 PCIBUS_readOneWord(Uint32 pciAddress, Uint32 *buf)
{
	return PCIBUS_read( buf, pciAddress, 1 );
}

/*----------------------------------------------------------------------------*/
/* You can call this function from a TSK only.
 * Do *NOT* call this function from the main(), or SWI (PRD, CLK), or HWI.
 */
IDEF Uint32 PCIBUS_writeOneWord(Uint32 pciAddress, Uint32 value)
{
	Uint32 buf = value;

	return PCIBUS_write( &buf, pciAddress, 1 );
}

/*---------------------------------------------------------------------------*/
IDEF Uint32 PCIBUS_getError(PCIBUS_AccessReqObj * pReqObj)
{
	return ((pReqObj->regPCIMC) & PCIBUS_TOE_MASK);
}

/*---------------------------------------------------------------------------*/
IDEF BOOL PCIBUS_isCompleted(PCIBUS_AccessReqObj * pReqObj)
{
	return (pReqObj->completed);
}

/*----------------------------------------------------------------------------*/
#endif /* USEDEFS */


#endif /* BSL_PCIBUS_SUPPORT */
#endif /* _BSL_PCIBUS_H_ */
/******************************************************************************\
* End of bsl_pcibus.h
\******************************************************************************/

